/*
 *      lark-eq-dialog.c
 *
 *      Copyright 2016-2018 Alex <alex@linuxonly.ru>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include "lark-eq-dialog.h"

enum {
	PROP_0,
	PROP_GSTREAMER,
	PROP_EQUALIZER_SETTINGS,
	LAST_PROP
};

static GParamSpec *properties [LAST_PROP];

struct _LarkEQDialog
{
	GtkDialog parent_instance;
	/*Objects*/
	LarkGstreamer *gstreamer;
	GSettings *eqsettings;
	/*Widgets*/
	GtkAdjustment *LarkEQ100Adjustment;
	GtkAdjustment *LarkEQ300Adjustment;
	GtkAdjustment *LarkEQ1KAdjustment;
	GtkAdjustment *LarkEQ3KAdjustment;
	GtkAdjustment *LarkEQ8KAdjustment;
	/*Used gain values*/
	gfloat eq100gain;
	gfloat eq300gain;
	gfloat eq1Kgain;
	gfloat eq3Kgain;
	gfloat eq8Kgain;
};

struct _LarkEQDialogClass
{
	GtkDialogClass parent_class;
};

G_DEFINE_TYPE (LarkEQDialog, lark_eq_dialog, GTK_TYPE_DIALOG)


static void lark_eq_dialog_save_button_clicked(GtkButton *button, LarkEQDialog *self)
{
	/*Save gain values*/
	g_settings_set_double(self->eqsettings, "gain-100hz", gtk_adjustment_get_value(self->LarkEQ100Adjustment));
	g_settings_set_double(self->eqsettings, "gain-300hz", gtk_adjustment_get_value(self->LarkEQ300Adjustment)); 
	g_settings_set_double(self->eqsettings, "gain-1khz", gtk_adjustment_get_value(self->LarkEQ1KAdjustment)); 
	g_settings_set_double(self->eqsettings, "gain-3khz", gtk_adjustment_get_value(self->LarkEQ3KAdjustment));
	g_settings_set_double(self->eqsettings, "gain-8khz", gtk_adjustment_get_value(self->LarkEQ8KAdjustment));
	
	gtk_widget_destroy(GTK_WIDGET(self));
}

static void lark_eq_dialog_close_button_clicked(GtkButton *button, LarkEQDialog *self)
{
	/*Restore initial gain values*/
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_100, self->eq100gain);
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_300, self->eq300gain);
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_1K,  self->eq1Kgain);
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_3K,  self->eq3Kgain);
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_8K,  self->eq8Kgain);
	
	gtk_widget_destroy(GTK_WIDGET(self));
}

static void lark_eq_dialog_gain_100_value_changed(GtkAdjustment *adjustment, LarkEQDialog *self)
{
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_100, gtk_adjustment_get_value(adjustment));
}

static void lark_eq_dialog_gain_300_value_changed(GtkAdjustment *adjustment, LarkEQDialog *self)
{
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_300, gtk_adjustment_get_value(adjustment));
}

static void lark_eq_dialog_gain_1K_value_changed(GtkAdjustment *adjustment, LarkEQDialog *self)
{
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_1K, gtk_adjustment_get_value(adjustment));
}

static void lark_eq_dialog_gain_3K_value_changed(GtkAdjustment *adjustment, LarkEQDialog *self)
{
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_3K, gtk_adjustment_get_value(adjustment));
}

static void lark_eq_dialog_gain_8K_value_changed(GtkAdjustment *adjustment, LarkEQDialog *self)
{
	lark_gstreamer_set_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_8K, gtk_adjustment_get_value(adjustment));
}

static void lark_eq_dialog_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
{
	LarkEQDialog *self = (LarkEQDialog *)object;
	
	switch (prop_id) {
		case PROP_GSTREAMER:
			g_value_set_pointer(value, self->gstreamer);
			break;
		case PROP_EQUALIZER_SETTINGS:
			g_value_set_pointer(value, self->eqsettings);
			break;
	}
}

static void lark_eq_dialog_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
{
	LarkEQDialog *self = (LarkEQDialog *)object;
	
	switch (prop_id) {
		case PROP_GSTREAMER:
			self->gstreamer = g_value_get_pointer(value);
			break;
		case PROP_EQUALIZER_SETTINGS:
			self->eqsettings = g_value_get_pointer(value);
			break;
	}
}

static void lark_eq_dialog_constructed(GObject *object)
{
	LarkEQDialog *self = (LarkEQDialog *)object;
	
	/*Save used gain values*/
	self->eq100gain = lark_gstreamer_get_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_100);
	self->eq300gain = lark_gstreamer_get_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_300);
	self->eq1Kgain  = lark_gstreamer_get_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_1K);
	self->eq3Kgain  = lark_gstreamer_get_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_3K);
	self->eq8Kgain  = lark_gstreamer_get_eq_gain(self->gstreamer, LARK_GSTREAMER_EQ_FREQ_8K);
	
	/*Set initial gain values*/
	gtk_adjustment_set_value(self->LarkEQ100Adjustment, self->eq100gain);
	gtk_adjustment_set_value(self->LarkEQ300Adjustment, self->eq300gain);
	gtk_adjustment_set_value(self->LarkEQ1KAdjustment,  self->eq1Kgain);
	gtk_adjustment_set_value(self->LarkEQ3KAdjustment,  self->eq3Kgain);
	gtk_adjustment_set_value(self->LarkEQ8KAdjustment,  self->eq8Kgain);
}

static void lark_eq_dialog_class_init(LarkEQDialogClass *klass)
{
	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
	GObjectClass *object_class = G_OBJECT_CLASS(klass);
	
	object_class->get_property = lark_eq_dialog_get_property;
	object_class->set_property = lark_eq_dialog_set_property;
	object_class->constructed = lark_eq_dialog_constructed;
	
	properties[PROP_GSTREAMER] =
		g_param_spec_pointer("gstreamer",
							"GStreamer object",
							"Pointer to GStreamer object",
							G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
							
	properties[PROP_EQUALIZER_SETTINGS] =
		g_param_spec_pointer("equalizer-settings",
							"Equalizer settings object",
							"Pointer to equalizer settings object",
							G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
								
	g_object_class_install_properties(object_class, LAST_PROP, properties);
	
	gtk_widget_class_set_template_from_resource(widget_class, "/ru/linuxonly/lark/ui/equalizer-dialog.ui");
	
	gtk_widget_class_bind_template_child(widget_class, LarkEQDialog, LarkEQ100Adjustment);
	gtk_widget_class_bind_template_child(widget_class, LarkEQDialog, LarkEQ300Adjustment);
	gtk_widget_class_bind_template_child(widget_class, LarkEQDialog, LarkEQ1KAdjustment);
	gtk_widget_class_bind_template_child(widget_class, LarkEQDialog, LarkEQ3KAdjustment);
	gtk_widget_class_bind_template_child(widget_class, LarkEQDialog, LarkEQ8KAdjustment);
	
	gtk_widget_class_bind_template_callback(widget_class, lark_eq_dialog_save_button_clicked);
	gtk_widget_class_bind_template_callback(widget_class, lark_eq_dialog_close_button_clicked);
	
	gtk_widget_class_bind_template_callback(widget_class, lark_eq_dialog_gain_100_value_changed);
	gtk_widget_class_bind_template_callback(widget_class, lark_eq_dialog_gain_300_value_changed);
	gtk_widget_class_bind_template_callback(widget_class, lark_eq_dialog_gain_1K_value_changed);
	gtk_widget_class_bind_template_callback(widget_class, lark_eq_dialog_gain_3K_value_changed);
	gtk_widget_class_bind_template_callback(widget_class, lark_eq_dialog_gain_8K_value_changed);
}

static void lark_eq_dialog_init(LarkEQDialog *self)
{
	gtk_widget_init_template(GTK_WIDGET(self));
}

GtkWidget *lark_eq_dialog_new(GtkWindow *parent, LarkGstreamer *gstreamer, GSettings *eqsettings)
{
	return g_object_new(lark_eq_dialog_get_type(), "transient-for", parent, "use-header-bar", TRUE, "gstreamer", gstreamer, "equalizer-settings", eqsettings, NULL);
}
